/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 1999 - 2005, Digium, Inc.
 *
 * Mark Spencer <markster@digium.com>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*! \file
 *
 * \author Grzegorz Sluja <grzegorz.sluja@genexis.eu>
 *
 * \ingroup channel_drivers
 * Implementation for speed dialing call numbers list
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "speeddial.h"
#include "asterisk/compiler.h"
#include "asterisk/logger.h"
#include "asterisk/dt_logger.h"
#define SPEEDDIAL_LIST_DUMP_DIR "/tmp/"

const char *speedDialFilename = "/nvram/voice/SpeedDialList";

SpeedDialCallList speedDialCallList;

int speeddial_read_from_file(SpeedDialCallList *storage) {
    FILE *file = fopen(speedDialFilename, "rb");

    if (!file) {
        ast_log(LOG_ERROR, "Failed to open file %s\n", speedDialFilename);
        return -1;
    }

    if (fread(storage, sizeof(SpeedDialCallList), 1, file) != 1 || storage->magic != SPEEDDIAL_MAGIC_VALUE)
    {
        fclose(file);
        ast_log(LOG_WARNING, "Invalid %s version or corrupted file. Resetting the speed dialing list\n", speedDialFilename);
        storage->magic = SPEEDDIAL_MAGIC_VALUE;
        storage->total_entries = 0;
        memset(storage->entries, 0, sizeof(storage->entries));

        return speeddial_write_to_file(storage);
    }
    fclose(file);

    return 0;
}

int speeddial_write_to_file(SpeedDialCallList *storage) {
    FILE *file = fopen(speedDialFilename, "wb");

    if (!file) {
        ast_log(LOG_ERROR, "Failed to open file %s\n", speedDialFilename);
        return -1;
    }

    if (fwrite(storage, sizeof(SpeedDialCallList), 1, file) != 1)
    {
        ast_log(LOG_ERROR, "Failed to write data to file %s\n", speedDialFilename);
        fclose(file);
        return -1;
    }
    fclose(file);

    return 0;
}

int speeddial_entry_exists(SpeedDialCallList *storage, const char *call_number) {
    for (int i = 0; i < storage->total_entries; i++) {
        if (storage->entries[i].is_valid && strcmp(storage->entries[i].call_number, call_number) == 0) {
            // Rearrange entry to the top if it exists and is not on top already
            if (i != (storage->total_entries -1)) {
                SpeedDialCallNumber temp = storage->entries[i];
                for (int j = i; j < storage->total_entries - 1; j++) {
                    storage->entries[j] = storage->entries[j + 1];
                }
                storage->entries[storage->total_entries - 1] = temp;
                speeddial_write_to_file(storage);
            }
            return 1;   // Entry exists
        }
    }

    return 0;  // Entry does not exist
}

int speeddial_add_entry(SpeedDialCallList *storage, const char *call_number) {
    size_t len = strlen(call_number);
    if (len < SPEEDDIAL_MIN_CALLNUMBER_SIZE || len > SPEEDDIAL_MAX_CALLNUMBER_SIZE) {
        ast_log(LOG_NOTICE, "Number [%s] has invalid lenght %zu, skip adding it to speed dialing list\n", call_number, len);
        return -1;
    }

    if (speeddial_entry_exists(storage, call_number))
        return 0;  // Do not add if it already exists

    // If storage is full, remove the oldest entry (FIFO)
    if (storage->total_entries == SPEEDDIAL_MAX_ENTRIES) {
		ast_log(LOG_NOTICE, "Speed dialing list is full, remove the oldest entry\n");
        // FIFO: Remove the oldest entry
        for (int i = 0; i < SPEEDDIAL_MAX_ENTRIES - 1; i++) {
            storage->entries[i] = storage->entries[i + 1];
        }
        storage->total_entries--;
    }

    // Add the new entry at the end
    strncpy(storage->entries[storage->total_entries].call_number, call_number, SPEEDDIAL_MAX_CALLNUMBER_SIZE);
    storage->entries[storage->total_entries].is_valid = 1;
    storage->total_entries++;

    speeddial_write_to_file(storage);

    return 1;
}

int speeddial_delete_entry(SpeedDialCallList *storage, const char *call_number) {
    for (int i = 0; i < storage->total_entries; i++) {
        if (storage->entries[i].is_valid && strcmp(storage->entries[i].call_number, call_number) == 0) {
            // Mark the entry as invalid
            storage->entries[i].is_valid = 0;

            // Shift all subsequent entries left to fill the gap if the deleted entry is not the last one
            for (int j = i; j < storage->total_entries - 1; j++) {
                storage->entries[j] = storage->entries[j + 1];
            }

            storage->total_entries--;
            speeddial_write_to_file(storage);
            return 1;
        }
    }

    return 0;
}

int speeddial_erase(SpeedDialCallList *storage) {
    storage->magic = SPEEDDIAL_MAGIC_VALUE;
    storage->total_entries = 0;
    memset(storage->entries, 0, sizeof(storage->entries));
    
    ast_log_dt(LOG_EVENT_CODE_V100);
	
    return speeddial_write_to_file(storage);
}

// Dump the current SpeedDial list to lst file in SPEEDDIAL_LIST_DUMP_DIR
int speeddial_dump_list(SpeedDialCallList *storage) {
    time_t now = time(NULL);
    struct tm *t = localtime(&now);
    char filename[256];
    strftime(filename, sizeof(filename), SPEEDDIAL_LIST_DUMP_DIR "speed_dial_%Y%m%d_%H%M%S.lst", t);

    FILE *file = fopen(filename, "w");
    if (!file) {
        ast_log(LOG_ERROR, "Failed to open file %s for writing\n", filename);
        return -1;
    }

    // Write the header
    fprintf(file, "Speed Dial List Dump - %04d-%02d-%02d %02d:%02d:%02d\n",
            t->tm_year + 1900, t->tm_mon + 1, t->tm_mday, t->tm_hour, t->tm_min, t->tm_sec);
    fprintf(file, "---------------------------------------------\n");

    // Write each valid speed dial entry to the file
    for (int i = 0; i < storage->total_entries; i++) {
        if (storage->entries[i].is_valid) {
            fprintf(file, "Entry %d: %s\n", i + 1, storage->entries[i].call_number);
        }
    }

    // Close the file
    fclose(file);

    ast_log(LOG_NOTICE, "Speed dial list dumped to file: %s\n", filename);

    return 0;
}
